package sun.misc;

import java.io.PrintStream;

public class RegexpPool
{
    private RegexpNode prefixMachine;
    private RegexpNode suffixMachine;
    private static final int BIG = Integer.MAX_VALUE;
    private int lastDepth;
    
    public RegexpPool() {
        this.prefixMachine = new RegexpNode();
        this.suffixMachine = new RegexpNode();
        this.lastDepth = Integer.MAX_VALUE;
    }
    
    public void add(String s, Object o) throws REException {
        this.add(s, o, false);
    }
    
    public void replace(String s, Object o) {
        try {
            this.add(s, o, true);
        }
        catch (Exception ex) {}
    }
    
    public Object delete(String s) {
        Object o = null;
        RegexpNode regexpNode = this.prefixMachine;
        int index = s.length() - 1;
        boolean n = true;
        if (!s.startsWith("*") || !s.endsWith("*")) {
            ++index;
        }
        if (index <= 0) {
            return null;
        }
        int index2 = 0;
        while (regexpNode != null) {
            if (regexpNode.result != null && regexpNode.depth < Integer.MAX_VALUE && (!regexpNode.exact || index2 == index)) {
            }
            if (index2 >= index) {
                break;
            }
            regexpNode = regexpNode.find(s.charAt(index2));
            ++index2;
        }
        RegexpNode regexpNode2;
        for (regexpNode2 = this.suffixMachine, ; --index >= 0 && regexpNode2 != null; regexpNode2 = regexpNode2.find(s.charAt(index))) {
            if (regexpNode2.result != null && regexpNode2.depth < Integer.MAX_VALUE) {
                n = false;
                regexpNode = regexpNode2;
            }
        }
        if (!(n == 0)) {
            if (s.equals(regexpNode.re)) {
                o = regexpNode.result;
                regexpNode.result = null;
            }
        }
        else if (s.equals(regexpNode.re)) {
            o = regexpNode.result;
            regexpNode.result = null;
        }
        return o;
    }
    
    public Object match(String s) {
        return this.matchAfter(s, Integer.MAX_VALUE);
    }
    
    public Object matchNext(String s) {
        return this.matchAfter(s, this.lastDepth);
    }
    
    private void add(String s, Object result, boolean b) throws REException {
        int i = s.length();
        RegexpNode regexpNode;
        if (s.charAt(0) == '*') {
            for (regexpNode = this.suffixMachine; i > 1; regexpNode = regexpNode.add(s.charAt(--i))) {}
        }
        else {
            boolean exact = false;
            if (s.charAt(i - 1) == '*') {
                --i;
            }
            else {
                exact = true;
            }
            regexpNode = this.prefixMachine;
            for (int j = 0; j < i; ++j) {
                regexpNode = regexpNode.add(s.charAt(j));
            }
            regexpNode.exact = exact;
        }
        if (regexpNode.result != null && !b) {
            throw new REException(s + " is a duplicate");
        }
        regexpNode.re = s;
        regexpNode.result = result;
        return;
    }
    
    private Object matchAfter(String s, int n) {
        RegexpNode regexpNode = this.prefixMachine;
        int beginIndex = 0;
        int endIndex = 0;
        int length = s.length();
        if (length <= 0) {
            return null;
        }
        int index = 0;
        while (regexpNode != null) {
            if (regexpNode.result != null && regexpNode.depth < n && (!regexpNode.exact || index == length)) {
                this.lastDepth = regexpNode.depth;
                beginIndex = index;
                endIndex = length;
            }
            if (index >= length) {
                break;
            }
            regexpNode = regexpNode.find(s.charAt(index));
            ++index;
        }
        RegexpNode regexpNode2;
        for (regexpNode2 = this.suffixMachine, ; --length >= 0 && regexpNode2 != null; regexpNode2 = regexpNode2.find(s.charAt(length))) {
            if (regexpNode2.result != null && regexpNode2.depth < n) {
                this.lastDepth = regexpNode2.depth;
                regexpNode = regexpNode2;
                beginIndex = 0;
                endIndex = length + 1;
            }
        }
        Object o = regexpNode.result;
        if (o != null && o instanceof RegexpTarget) {
            o = ((RegexpTarget)o).found(s.substring(beginIndex, endIndex));
            return o;
        }
        return o;
    }
    
    public void reset() {
        this.lastDepth = Integer.MAX_VALUE;
    }
    
    public void print(PrintStream printStream) {
        printStream.print("Regexp pool:\n");
        if (this.suffixMachine.firstchild != null) {
            printStream.print(" Suffix machine: ");
            this.suffixMachine.firstchild.print(printStream);
            printStream.print("\n");
        }
        if (this.prefixMachine.firstchild != null) {
            printStream.print(" Prefix machine: ");
            this.prefixMachine.firstchild.print(printStream);
            printStream.print("\n");
        }
    }
}
